ad_library {

    Provide functions for Content Repository folders
    
    @author Jun Yamog
    @creation-date 10 Aug 2002
    @cvs-id $Id: bcms-folder-procs.tcl,v 1.15 2004/03/30 14:04:21 jeffd Exp $
    @cvs-id $Id: bcms-folder-procs.tcl,v 1.15 2004/03/30 14:04:21 jeffd Exp $
}

namespace eval bcms::folder {}


ad_proc -public bcms::folder::get_cr_root_folder {} {
    Gets the CR Pages root, this is helpful as BCMS will always 
    create its root from there. 

    @return the folder_id of cr_folder's Pages root
} {
    return [db_string get_cr_root_folder {}]
}


ad_proc -public bcms::folder::get_bcms_root_folder {} {
    @return the BCMS folder_id based from the current package instance
    if it is not setup properly it returns 0
} {
    set bcms_root_folder [parameter::get -parameter root_folder_id -default 0]
    if {($bcms_root_folder > 0) || ($bcms_root_folder == [bcms::folder::get_cr_root_folder])} {
        return $bcms_root_folder
    } else {
        ns_log warning "bcms root folder id not properly set, please set on the admin pages"
        return 0
    }
}


ad_proc -public bcms::folder::create_folder {
    {-name:required}
    {-folder_label:required}
    {-parent_id:required}
    {-description ""}
    {-package_id ""}
    {-context_id ""}
    {-folder_id ""}
    {-subtypes:boolean}
    {-content_types ""}
} {
    creates a new folder in cr_folders

    @param name           name or url of the folder
    @param folder_label   the label used for the folder, this is the human readable title of 
                          the folder
    @param parent_id      the folder_id of which this folder will become a subfolder
    @param description    a description for this folder

    @param content_types  If specified, will register the given content types to the folder.
    @param subtypes       If set, will cause the content type's subtypes to also be registered to this folder
    
    @return the folder_id that was created
} {
    if { [empty_string_p $context_id] } {
        set context_id $parent_id
    }                

    set extra_vars [ns_set create]
    ns_set put $extra_vars label $folder_label
    oacs_util::vars_to_ns_set \
        -ns_set $extra_vars \
        -var_list { name parent_id description context_id folder_id package_id }

    db_transaction {
        set folder_id [package_instantiate_object \
                           -extra_vars $extra_vars \
                           -package_name "content_folder" \
                           "content_folder"]

        # Set package_id. For some reason, this is not included in the PL/SQL call
        if { ![empty_string_p $package_id] } {
            db_dml update_package_id { 
                update cr_folders set package_id = :package_id where folder_id = :folder_id 
            }
        }
        
        # Register folders
        if { [llength $content_types] > 0 } {
            bcms::folder::register_content_types \
                -folder_id $folder_id \
                -subtypes=$subtypes_p \
                -content_types $content_types
        }                                             
    }
    ns_log notice "bcms::folder::create_folder created folder $folder_id"
    return $folder_id
}


# for possible contribution - maybe we move this on CR's api
ad_proc -public bcms::folder::delete_folder { 
    {-folder_id:required} 
    {-cascade:boolean}
} {
    deletes a cr_folder based from the folder_id 

    @param folder_id of folder you wish to delete
    
    @return 1 if its able to delete the folder otherwise
    it returns 0
} {
    # TODO: test of we are catching the error message
    if [catch {db_exec_plsql delete_folder {}} folder_id] {
        ns_log warning "bcms::folder::delete_folder unable to delete $folder_id"
        return 0
    } else {
        return 1
        ns_log notice "bcms::folder::delete_folder deleted folder $folder_id"
    }

}

ad_proc -public bcms::folder::set_folder {
    {-folder_id:required}
    {-name}
    {-label}
    {-parent_id}
    {-description}
} {
    set/edits the folder properties.

    @param folder_id    folder_id of the folder you wish to edit
    @param name         edit the name of the folder
    @param label        edit the label of the folder
    @param parent_id    edit or basically move this folder to another folder
} {

    set update_list [list]

    if [info exists name] {
        lappend update_list "name = :name"
    }
    if [info exists parent_id] {
        lappend update_list "parent_id = :parent_id"
    }

    if {[llength $update_list] > 0} {
        set update_string [join $update_list ","]
        db_dml edit_folder {}
    }

    if [info exists label] {
        db_dml edit_folder_label {}
    }
    if [info exists description] {
	db_dml edit_folder_desc {update cr_folders set description=:description}
    }

    ns_log notice "bcms::folder::set_folder updated $folder_id"

    
}


ad_proc -public bcms::folder::get_folder {
    {-folder_id:required}
} {
    gets the properties of a folder

    @param folder_id you wish to get

    @return an array with the following elments:  
    parent_id, name, label, description, has_child_folders, has_child_symlinks, package_id
    if no folder is found a blank array is returned
} {
    if {[db_0or1row get_1folder {} -column_array one_folder]} {
        return [array get one_folder]
    } else {
        return {}
    }

}


ad_proc -public bcms::folder::list_folders {
    {-parent_id}
    {-multirow_name bcms_list_folders}
    {-return_list:boolean false}
} {
    returns a listing of the content folder under the parent_id, if
    no parent_id is provided the bcms root folder is used as the parent_id

    @param parent_id         lists the folders underneath this folder_id
    @param multirow_name     you can supply your own multirow name
    @param return_list       if supplied a list of ns_sets otherwise it
                             returns a multirow uses the multirow_name if given

    @return this columns: folder_id, name, parent_id, label, has_child_folders,
    has_child_symlinks, package_id
} {

    if ![info exists parent_id] {
        # if the parent_id does not exists then start from the root
        set parent_id [get_bcms_root_folder]
    }

    if $return_list_p {
        # return a list of lists
        return [db_list_of_ns_sets get_cr_folders {}]
    } else {
        # make a multirow
        db_multirow $multirow_name get_cr_folders {}
    }
}


ad_proc -public bcms::folder::tree_folders {
    {-parent_id}
    {-prepend_path "./"}
    {-multirow_name bcms_list_folders}
    {-return_list:boolean false}
    {-include_parent:boolean}
} {
    similar to list_folders but also gives the subfolders underneath it
    gives some more addtional columns 

    @param parent_id         gets the subfolder underneath this folder_id
    @param prepend_path      you can add a string to prepend the path column
    @param multirow_name     you can supply your own multirow name
    @param return_list       if supplied it returns a list of ns_sets otherwise it
                             returns a multirow uses the multirow_name if given
    @param include_parent    if supplied, it will also include info about the parent

    @return this columns: folder_id, name, parent_id, label, has_child_folders,
    has_child_symlinks, package_id, level, parent_level, path

} {
    #TODO we may need to optimze getting the path, we are currently using
    #content_item__get_path (which is not that fast)


    if ![info exists parent_id] {
        # if the parent_id does not exists then start from the root
        set parent_id [get_bcms_root_folder]
    }

    if {[info exists include_parent_p]} {
        set addtnl_where ""
    } else {
        set addtnl_where "and i.item_id <> p.item_id"
    }

    if $return_list_p {
        # return a list of lists
        return [db_list_of_ns_sets get_cr_folders {}]
    } else {
        # make a multirow
        db_multirow $multirow_name get_cr_folders {}
    }

}


ad_proc -public bcms::folder::register_content_types {
    {-subtypes:boolean}
    {-folder_id:required}
    {-content_types:required}
} {
    Register one or more content types to a folder.
    
    @param subtypes Whether or not we should also register the content type's subtypes.
} {
    set subtypes_p [db_boolean $subtypes_p]
    foreach content_type $content_types {
        db_exec_plsql register_type {}
    }
}

ad_proc -public bcms::folder::is_registered_content_type {
    {-folder_id:required}
    {-content_type:required}
} {
    checks to see if a content type is registered on a folder

    @param folder_id      id of the folder to check
    @param content_type   content type to check if its registered on not

    @return 1 if its registered if not 0
} {
    set result [db_exec_plsql check_content_type {}]
    if {$result == "t"} {
        return 1
    } else {
        return 0
    }
}

ad_proc -public bcms::folder::get_id_by_package_id {
    {-package_id ""}
    {-parent_id ""}
} {
    Get the ID of the folder with the given package_id, and optionally with the given parent.

    @return folder_id or empty string if no such folder.
} {
    if { [empty_string_p $package_id] } {
        set package_id [ad_conn package_id]
    }

    if { [empty_string_p $parent_id] } {
        return [db_string select_folder_id {} -default {}]
    } else {
        return [db_string select_folder_id_with_parent_id {} -default {}]
    }
}

ad_proc -public bcms::folder::content_types_select {
	{-folder_id:required}
} {
	Returns a list of lists of pretty name and content type
	suitable for a form select widget
} {

	return [db_list_of_lists get_valid_types "select pretty_name, content_type from cr_folder_content_types where folder_id=:folder_id"]
	
}
