ad_library {

	Procs for Content Repository content items

	@author Jun Yamog
	@creation-date 10 Aug 2002
	@cvs-id $Id: bcms-item-procs.tcl,v 1.17 2004/04/20 21:12:19 jeffd Exp $

}

namespace eval bcms::item {}

# for possible contribution
ad_proc -public bcms::item::get_content_type {
    {-item_id:required}
} {
    returns the content type of a particular item_id

    @param item_id you wish to know what content type

    @return the content type of an item
} {
    array set one_item [get_item -item_id $item_id]
    return $one_item(content_type)
}


ad_proc -public bcms::item::create_item {
    {-item_id ""}
    {-item_name:required}
    {-parent_id:required}
    {-content_type "content_revision"}
    {-storage_type "text"}
    {-creation_user}
    {-creation_ip}
} {
    This proc only creates the content item but not the revision.  
    Normally you would like couple this with create_revision or 
    your own custom create_revision.

    @param item_id           Optionally supply an item_id.
    @param item_name         the name or url that will be used for the content item
    @param parent_id         the parent_id which the page will be under, normally this is a folder
    @param content_type      content type of the item to be created
    @param storage_type      where to store the item, valid values are: text, file and lob (lob support not yet implemented)
    @param creation_user  user_id creating this item
    @param creation_ip  ip address which this item is created

    @return the item_id that was created
} {
    if {![info exists creation_user]} { set creation_user [ad_conn user_id] }
    if {![info exists creation_ip]} { set creation_ip [ad_conn peeraddr] }

    set item_id [db_exec_plsql create_item {}]
    ns_log Debug "bcms::item::create_item created new item $item_id"
    return $item_id
}

ad_proc -public bcms::item::create_link {
    {-parent_id:required}
    {-target_id:required}
    {-name:required}
    {-label:required}
    {-creation_user}
    {-creation_ip}
} {
    creates a symlink to the content item target_id

    @param parent_id     the item which will hold the symlink, normally the destination folder
    @param target_id     link to which item_id, the source
    @param name          name of the link
    @param label         label or title of the link
    @param creation_user  user_id creating this item
    @param creation_ip  ip address which this item is created

    @return the item_id of the new link
} {
    if {![info exists creation_user]} { set creation_user [ad_conn user_id] }
    if {![info exists creation_ip]} { set creation_ip [ad_conn peeraddr] }

    set item_id [db_exec_plsql create_link {}]
    ns_log Debug "bcms::item::create_link created new link $item_id to $target_id"
    return $item_id
}


ad_proc -public bcms::item::set_item {
    {-item_id:required}
    {-name}
    {-parent_id}
} {
    edits the content item's name or folder/parent_id

    @param item_id     id of item to edit
    @param name        if supplied it will edit the name
    @param parent_id   if supplied it will move the item to the new parent
} {

    if [info exists name] {
        db_dml update_name {}
    }
    if [info exists parent_id] {
        db_exec_plsql update_parent {}
    }

}


ad_proc -public bcms::item::delete_item {
    -item_id:required
} {
    nukes out the content item.  
    This also deletes the all the revision of the content item

    @param item_id of the item to be deleted

    @return 0 if successful.  
} {
    return [db_exec_plsql delete_item {}]
}


ad_proc -public bcms::item::list_items {
    -parent_id:required
    {-revision latest}
    {-multirow_name bcms_list_items}
    {-return_list:boolean false}
    {-orderby "item_id asc"}
    {-show_only}
} {
    Gets all content item under the parent_id

    @param parent_id            id of the which contents you wish to list, normally a folder
    @param revision             returns items that have live or latest revision under the parent_id
    @param multirow_name        you can supply your own multirow name
    @param return_list          if supplied a list of ns_sets otherwise it
                                returns a multirow uses the multirow_name if given
    @param show_only            ids of the item to show only, normally this used in conjuction with
                                list builder and paginator
     
    @return rows of content items below the parent_id
    with the following columns:

    item_id, name, live_revision, latest_revision, publish_status, 
    content_type, storage_type, tree_sortkey, last_modified, title, description

} {
    # TODO: make use of db_map... 

    # initialize the addtionals we will use
    set addtnl_where_list [list]
    set addtnl_where_folder_list [list]
    set addtnl_select_list [list]

    # if additional revision data must be returned, set the additional columns here
    # for revisions.
    set addtnl_from ""

    switch -exact $revision {
        latest { lappend addtnl_where_list "i.latest_revision = r.revision_id" }
        live { lappend addtnl_where_list "i.live_revision = r.revision_id" }
        default { 
            # abort the proc
            ns_log debug "bcms: revision requested not valid" 
            error "revision requested not valid"
            return
        }
    }

    # show only a subset of item_id
    if [info exists show_only] {
        lappend addtnl_where_list "i.item_id in ($show_only)"
        lappend addtnl_where_folder_list "i.item_id in ($show_only)"
    }

    # construct the additionals
    if {[llength $addtnl_where_list] > 0} {
        set addtnl_where "and [join $addtnl_where_list " and "]"
    } else {
        set addtnl_where ""
    }
    if {[llength $addtnl_where_folder_list] > 0} {
        set addtnl_where_folder "and [join $addtnl_where_folder_list " and "]"
    } else {
        set addtnl_where_folder ""
    }
    if {[llength $addtnl_select_list] > 0} {
        set addtnl_select_folder ", null as [join $addtnl_select_list ", null as "]"
        set addtnl_select_list [linsert $addtnl_select_list 0 r.title r.description]
        set addtnl_select ", [join $addtnl_select_list ", "]"
    } else {
        set addtnl_select_folder ""
        set addtnl_select_list [linsert $addtnl_select_list 0 r.title r.description]
        set addtnl_select ", [join $addtnl_select_list ", "]"
    }

    if {$return_list_p} {
        # return a list of lists since its not easy to pass multirow from proc to proc
        return [db_list_of_ns_sets get_items {}]
    } else {
        # make a multirow, it is likely that this proc is not being called from another proc
        db_multirow -extend {last_modified_pretty} $multirow_name get_items {} {
	set last_modified_pretty "[lc_time_fmt $last_modified "%q %r"]"
        }
    }

}

ad_proc -public bcms::item::tree_items {
    {-parent_id:required}
    {-revision}
    {-content_type}
    {-level_diff 0}
    {-multirow_name}
    {-return_list:boolean false}
} {
    this is similar to list_items, i may merge it later on.  but since
    both are complex i haven't decided if making them into one and
    adding more complexity to list_items is good, once this proves to be ok and
    fast enough then I will merge the 2 api.

    TODO: it seems i am not using this lately must review and to look more like
    bcms::folder::tree_folders
} {
    # initialize the addtional where we will use
    set addtnl_where_list [list]
    set addtnl_select_list [list]

    # if additional revision data must be returned, set the additional columns here
    # for revisions.  we need to make use of db_map
    if [info exists revision] {
        lappend addtnl_select_list a.revision_id "case when o.content_type = 'content_folder' then content_folder__get_label(o.item_id) else a.title end as title" a.description a.publish_date
        set addtnl_inner_select ", r.revision_id, r.title, r.description, r.publish_date "
        set addtnl_from ", cr_revisions r"
        switch -exact $revision {
            latest { lappend addtnl_where_list "i.latest_revision = r.revision_id" }
            live { lappend addtnl_where_list "i.live_revision = r.revision_id" }
            default { 
                # abort the proc
                ns_log debug "bcms: revision requested not valid" 
                return
            }
        }
    } else {
        set addtnl_inner_select ""
        set addtnl_from ""
    }

    # TODO: make use of db_map... 
    # check to see how deep we want to go
    if {$level_diff != 0} {
        lappend addtnl_where_list ":level_diff >= tree_level(i.tree_sortkey) - tree_level(p.tree_sortkey)"
    }

    # pick a particular content type
    if [info exists content_type] {
        lappend addtnl_where_list "content_type = :content_type"
    }

    # construct the additionals
    if {[llength $addtnl_where_list] > 0} {
        set addtnl_where "and [join $addtnl_where_list " and "]"
    } else {
        set addtnl_where ""
    }
    if {[llength $addtnl_select_list] > 0} {
        set addtnl_select ", [join $addtnl_select_list ", "]"
    } else {
        set addtnl_select ""
    }

    if {$return_list_p} {
        # return a list of lists since its not easy to pass multirow from proc to proc
        return [db_list_of_ns_sets get_items {}]
    } else {
        # make a multirow, it is likely that this proc is not being called from another proc
        if ![info exists multirow_name] { set multirow_name bcms_list_items }
        db_multirow $multirow_name get_items {}
    }
}

ad_proc -public bcms::item::list_related_items {
    {-item_id:required}
    {-relation_tag:required}
    {-multirow_name bcms_list_related_items}
    {-revision}
    {-return_list:boolean false}
} {
    get the related items of item_id.  

    @param item_id        the item you want to check the relations with
    @param relation_tag   what type of relationship to the item you want it to list
    @param multirow_name  you can supply your own multirow name
    @param return_list    if supplied a list of ns_sets otherwise it
                          returns a multirow uses the multirow_name if given
    @param revision       will only returns items that have live or latest revision under the parent_id
                          if this is used additional info are returned such as revision_id, title, description, publish_date

    @return the following columns

    item_id, name, live_revision, latest_revision, publish_status, 
    content_type, storage_type, rel_id, order_n, path

} {
    # initialize the addtional where we will use
    set addtnl_where_list [list]

    # if additional revision data must be returned, set the additional columns here
    # for revisions.  we need to make use of db_map
    if [info exists revision] {
        set addtnl_select ", r.revision_id, r.title, r.description, r.publish_date, r.mime_type"
        set addtnl_from ", cr_revisions r"
        switch -exact $revision {
            latest { lappend addtnl_where_list "i.latest_revision = r.revision_id" }
            live { lappend addtnl_where_list "i.live_revision = r.revision_id" }
            default { 
                # abort the proc
                ns_log debug "bcms: revision requested not valid" 
                return
            }
        }
    } else {
        set addtnl_select ""
        set addtnl_from ""
    }

    # construct the additional where
    if {[llength $addtnl_where_list] > 0} {
        set addtnl_where "and [join $addtnl_where_list " and "]"
    } else {
        set addtnl_where ""
    }

    if {$return_list_p} {
        # return a list of lists since its not easy to pass multirow from proc to proc
        return [db_list_of_ns_sets get_related_items {}]
    } else {
        # make a multirow, it is likely that this proc is not being called from another proc
        db_multirow $multirow_name get_related_items {}
    }

}

ad_proc -public bcms::item::get_item_id_by_url {
    {-root_id:required}
    {-url:required}
    {-resolve_index:boolean}
} {
    get the content item details based from the url. 

    @param root_id            starting from which url to resolve the item
    @param url                the url which we would like to resolve
    @param resolve_index      return the item_id of the index content item on a content_folder
                              rather than the content_folder's item_id

    @return the item_id of the item from that URL, or the empty string if no such item exists.
} {
    set resolve_index [db_boolean $resolve_index_p]
    return [db_exec_plsql get_item_id {}]
}


#TODO: use the views to select the revision info
ad_proc -public bcms::item::get_item_by_url {
    {-root_id:required}
    {-url:required}
    {-revision ""}
    {-resolve_index:boolean}
} {
    get the content item details based from the url. 

    @param root_id            starting from which url to resolve the item
    @param url                the url which we would like to resolve
    @param revision           if "-revision live" or "-revision lastest" is given then additional
                              properties are returned
    @param resolve_index      return the item_id of the index content item on a content_folder
                              rather than the content_folder's item_id

    @return an array that contains the following  
    item_id, name, parent_id, live_revision, latest_revision, publish_status, content_type 
    and storage_type

    if "-revision live" or "-revision lastest" is given then the details of the
    live or lastest revision is also place inside the array.  The additional
    details are:

    revision_id, title, content, description, publish_date, mime_type

    if it returns a single element array that means no item exists so using
    [array size yourarray] > 0 should check if such item does exists

    [array size yourarray] > 1 should check if a live revision was retrieved.  item exists
    but no live revision
    
    Be careful in checking [array size yourarray] inside a loop, you must unset the array first.
    array unset yourarray.

} {
    set item_id [get_item_id_by_url -root_id $root_id -url $url -resolve_index=$resolve_index_p]

    if { [string equal $item_id ""] } {
        return {}
    }

    array set item [get_item -item_id $item_id -revision $revision]
    # we need to see if we got any item, if not there is an item but the revision is not live
    # since we have a item_id
    if {[array size item] > 0} {
        return [array get item]
    } else {
        return [list item_id $item_id]
    }
}

#TODO: use the views to select the revision info
ad_proc -public bcms::item::get_item {
    {-item_id:required}
    {-revision ""}
} {
    get the content item details. 

    @param item_id            id of the item you want to get properties
    @param revision           if "-revision live" or "-revision lastest" is given then additional
                              properties are returned

    @return an array that contains the following  
    item_id, name, parent_id, live_revision, latest_revision, publish_status, content_type,
    storage_type

    if "-revision live" or "-revision lastest" is given then the details of the
    live or lastest revision is also place inside the array.  The additional
    details are:

    revision_id, title, content, description, publish_date, mime_type

} {
    # initialize the addtional where we will use
    set addtnl_where_list [list]

    # currently I have implemented it by building the query as opposed to using 3
    # queries.  If I get some complains regarding the readability then I will
    # just make use of 3 separate queries rather than building them
    # the api should insulate other developers on how I implement this.

    # if additional revision data must be returned, set the additional columns here
    # for revisions.  I am not using db_map since I think its not useful.
    if { [exists_and_not_null revision] } {
        set addtnl_select ", r.revision_id, r.title, r.content, r.description, r.publish_date, r.mime_type"
        set addtnl_from ", cr_revisions r"
        switch -exact $revision {
            latest { lappend addtnl_where_list "i.latest_revision = r.revision_id" }
            live { lappend addtnl_where_list "i.live_revision = r.revision_id" }
            default { 
                ns_log debug "bcms: revision requested not valid" 
                return {}
            }
        }
    } else {
        set addtnl_select ""
        set addtnl_from ""
    }

    # construct the additional where
    if {[llength $addtnl_where_list] > 0} {
        set addtnl_where "and [join $addtnl_where_list " and "]"
    } else {
        set addtnl_where ""
    }

    if {[db_0or1row get_1item {} -column_array one_item]} {
        return [array get one_item]
    } else {
        return {}
    }
}


ad_proc -public bcms::item::relate_item {
    {-relation_tag:required}
    {-item_id:required}
    {-related_object_id:required}
    {-order_n ""}
    {-relation_type "cr_item_rel"}
} {
    relate item_id to related_object_id with relation_type

    @param relation_tag        what relationship tag item_id and related_object_id will have, you have to register the relation_tag
                               between the object type of item_id and related_object_id if they are custom content types
    @param item_id             id of the item you wish to put a relation to.
    @param related_object_id   id of the item/object you wish to have relation with.
    @param order_n             order of the relation, you can have 1 to many relation.  the order of this relation.
    @param relation_type LARS HELP NEEDED

    @return LARS HELP NEEDED

} {
    return [db_exec_plsql relate_item {}]
}

ad_proc -public bcms::item::unrelate_item {
    {-rel_id:required}
} {
    unrelate an item

    @param rel_id   the relationship id you wish to break/unrelate
} {
    return [db_exec_plsql unrelate_item {}]
}


# TODO: remove this?  too pg specific?
ad_proc -public bcms::item::get_sortkeys {
    {-item_id:required}
} {
    gets the tree_sortkey of an item. current_sortkey, next_sortkey and prev_sortkey
    its postgres specific so i need to rehash this to be portable to oracle
} {

    array set sortkeys {}

    db_1row get_currentkey {}
    set sortkeys(current_sortkey) $current_sortkey

    set sortkeys(next_sortkey) [db_string get_nextkey {} -default {}]
    set sortkeys(prev_sortkey) [db_string get_prevkey {} -default {}]

    return [array get sortkeys]

}


ad_proc -public bcms::item::format_string_as_url {
	{-string:required}
    {-truncate 10}
} {
	trim the string to format as url

    @param string     the string to format
    @param truncate   if the string is long truncate at what point

    @return the url friendly transformed string
} {
	set string [string trim $string]
    regsub -all " " $string "" string
   	regsub -all {([^a-zA-Z0-9])} $string "" string
    
    
    return [string range [string tolower $string] 0 $truncate]
}

ad_proc -public bcms::item::is_item_duplicate_p {
    {-url:required}    
    {-root_id:required}
    {-item_id 0}
} {
	checks if given url is a duplicate

    @param url         url to check
    @param root_id     where to start traversing the url
    @param item_id     id of the item being checked
        
    @return 1 if duplicate 0 if not
} {	
    set duplicate_p 0
  	array set existing_item [bcms::item::get_item_by_url -url $url -root_id $root_id]

    if {([array size existing_item] > 0)} {
        if {($existing_item(item_id) != $item_id)} {
            set duplicate_p 1
        }
    }

    return $duplicate_p 
}

ad_proc -public bcms::item::create_unique_url {
    {-url:required}    
    {-root_id:required}
    {-count 10}
} {
	this eliminates the chance of having dupplicate pages
    it checks if url exists, if it does it then pads a number after url

    @param url      the url to check
    @param root_id  start the checking from which root_id/folder_id
    @param count    pad a number up to how many times if there is a duplicate

    @return a none duplicate url

} {
    set url [bcms::item::format_string_as_url -string $url]

	set ctr 0  	
   	while { ([bcms::item::is_item_duplicate_p -url $url -root_id $root_id] > 0) && ($ctr <= $count) } {
        set url "${url}1"          
        set ctr [expr $ctr + 1]           
   	}        
    if {$ctr > $count} {
        error "too many duplicate names/url on this folder. 
        either increase the count param or pass in more unique name/url"
    }
    return $url        
}


ad_proc -public bcms::item::get_item_path {
    {-item_id:required}
    {-root_id}
    {-prepend_path ''}
    {-multirow_name bcms_item_path}
    {-return_list:boolean}
    {-no_parent:boolean}
    {-root_title ""}
} {
    returns a multirow/list of path from the root_id to the item_id.  

    @param item_id           build the path to this item
    @param root_id           build the path from this folder, if not supplied it use the current bcms root
    @param prepend_path      you can add a string to prepend the path column
    @param multirow_name     you can supply your own multirow name
    @param return_list       if supplied a list of ns_sets otherwise it
                             returns a multirow uses the multirow_name if given
    @param no_parent         does not return additional rows containing the parents to this item

    @return a multirow/list with following elements item_id, path, title
} {
    if ![info exists root_id] {
        # if the parent_id does not exists then start from the root
        set root_id [bcms::folder::get_bcms_root_folder]
    }

    if {$no_parent_p} {
        set addtnl_where "and i.item_id = :item_id"
    } else {
        set addtnl_where ""
    }

    if {$return_list_p} {
        return [db_list_of_ns_sets get_item_path {}]
    } else {
        db_multirow $multirow_name get_item_path {}
    }
}


ad_proc -public bcms::item::assign_category {
   {-category_id:required}
   {-item_id:required}
} {
    categorize a content item to a category

    @param category_id    The keyword id of the category we map to
    @param item_id        The item id of the content item we map

} {
    return [db_exec_plsql assign_category {}]
}


ad_proc -public bcms::item::unassign_category {
   {-category_id:required}
   {-item_id:required}
} {
    removes map of a category to a content item

    @param category_id     The category id of the category we unmap to
    @param item_id         The item id of the content item we unmap

} {
    return [db_exec_plsql unassign_category {}]
}


ad_proc -public bcms::item::clear_categories {
    {-item_id:required}
} {
    clears all the assigned categories for a particular item

    @param item_id        The item id of the content item we unmap
} {
    db_list unassign_all_category {}
}


# TODO: make this similar to list_X calls?
ad_proc -public bcms::item::get_categories {
    {-item_id:required}
} {
    get the categories assigned to this item

    @param item_id           id of the item 
    
    @return category ids assigned to this item
} {

    return [db_list get_categories {}]
}
