ad_library {

    Classified Ads Procs (Containers)

    @author Deds Castillo (deds@infiniteinfo.com)
    @creation-date 2002-10-08
    @cvs-id $Id: ads-procs.tcl,v 1.12 2004/02/26 15:03:08 jeffd Exp $

}

namespace eval classified-ads::ads {}

ad_proc -public classified-ads::ads::new {
	{-item_id_element:required}
	{-form_id:required}
	{-category_id}
} {
	Creates a new ad

	@author Deds Castillo
	@author Roberto Mello

	@param form_id The ID of the form that contains the name/value pairs
	@param item_id_element The name of the form element that should be used as the item id
	@param category_id The category of this ad.

} {
	if { [empty_string_p $category_id] } {
		# 
		# No category passed. Assign to package_id
		#
		set package_id [ad_conn package_id]
		set category_id [get_folder_id -package_id $package_id]
	}

	return [classified-ads::create_ca_item -item_id_element $item_id_element \
			-content_type "ca_ad" \
			-form_id $form_id \
			-category_id $category_id \
		]
}


ad_proc -public classified-ads::ads::edit {
	{-item_id_element:required}
	{-form_id:required}
} {
	Edits properties of an existing ad

	@param form_id The ID of the form that contains the name/value pairs
	@param item_id_element The name of the form element that should be used as the item id

	@see classified-ads::edit_ca_item

} {
	return [classified-ads::edit_ca_item \
			-item_id_element $item_id_element \
			-form_id $form_id \
		]
}


ad_proc -public classified-ads::ads::delete {
	{-ad_id:required}
} {
	delete an existing ad

	@param ad_id The item id of the ad to delete
} {
	db_transaction {
		db_exec_plsql delete_ad {}
		db_exec_plsql delete_content {}
	}
	
	return 0
}

ad_proc -public classified-ads::ads::get {
	{-ad_id:required}
	{-revision_id ""}
	{-array:required}
} {
	Gets the data of a particular ad

	@author Deds Castillo
	@author Roberto Mello

	@param item_id The ID ad to get
	@param revisionid If present, then get the data for this revision. If not get the data for the latest revision
	@param array The array where the data is stored
} {
	upvar $array row

	# 
	# Get some necessary information
	# 

        set package_id [ad_conn package_id]
	set content_type "ca_ad"

	if {[empty_string_p $revision_id]} {
	    set revision_id [classified-ads::get_latest_revision -item_id $ad_id]
	}

	set keyword_id [classified-ads::ads::get_category_id -ad_id $ad_id]

	#
	# object_id, object_type, context_id, security_inherit_p, creation_user, 
	# creation_date, creation_ip, last_modified, modifying_user, modifying_ip, 
	# tree_sortkey, revision_id, title, item_id, description, publish_date, 
	# mime_type, nls_language, name, parent_id, ad_id
	#
	db_1row select_data {} -column_array row
	
	set row(pretty_publish_date) [util_AnsiDatetoPrettyDate $row(publish_date)]
	set row(data) [classified-ads::get_content -revision_id $revision_id]
	set row(keyword_id) $keyword_id 
	
	return 0
}


ad_proc -public classified-ads::ads::get_assigned_ads_by_category {
	{-category_id ""}
	{-limit 0}
} {
	Gets all the ads assigned to a category (or all categories in a package.)

	@param category_id The system returns all ads mapped to these categories, or
 			   all ads if no category is passed.
	@param limit The limit of rows to return. 0 means return all rows.
	
	@author Roberto Mello
} {
	set parent_id [classified-ads::get_folder_id -package_id [ad_conn package_id]]

	#
	# If no category passed, use the package category
	#

	if {[empty_string_p $category_id]} {
	    set category_id [classified-ads::categories::get_package_keyword_id]
	}

	if { $limit == 0 } {
		set limit_clause ""
	} else {
		set limit_clause "LIMIT $limit"
	}

	#
	# Get custom fields for this category.
	#

	set custom_fields_list [util_memoize "classified-ads::categories::get_custom_fields -keyword_id $category_id"]
	set custom_fields ""

	if {[llength $custom_fields_list] > 0} {
		foreach field $custom_fields_list {
			append custom_fields ", ads.[lindex $field 0]"
		}
	}

	return [db_list_of_ns_sets select_ads {}]
}


ad_proc -public classified-ads::ads::get_unassigned_ads_by_category {
	{-category_id_list ""}
	{-limit 0}
} {
	Gets all the ads NOT assigned to the categories passed.

	@param container_id_list A list of category ids. The system returns all 
	       ads not mapped to these categories.
	@param limit The limit of rows to return. 0 means return all rows.

} {
	# DEDS: FIXME - this uses a subselect.  this will get inefficient
	#	       when faced with a large db.  optimize this.
	set condition_stub ""

	if {[llength $category_id_list]} {
	    set condition_stub "where keyword_id in ([join $category_id_list ", "])"
	}

	set parent_id [classified-ads::get_folder_id -package_id [ad_conn package_id]]

	if { $limit == 0 } {
		set limit_clause ""
	} else {
		set limit_clause "LIMIT $limit"
	}

	return [db_list_of_ns_sets select_ads {}]
}

ad_proc -public classified-ads::ads::get_category_id {
	{-ad_id:required}
} {
	Return the keyword_id of the category the passed ad belongs to.

	@author Roberto Mello
} {
	if { [db_0or1row get_category_id {}] } {
		return $keyword_id
	} else {
		return 0
	}
}

ad_proc -public classified-ads::ads::generate_trail {
	{-ad_id:required}
	{-append_element}
} {
	Generates a cookie trail for a particular ad.

	@param ad_id The id of the ad.
	@author Roberto Mello
} {
	set keyword_id [classified-ads::ads::get_category_id -ad_id $ad_id]
	
	if { [exists_and_not_null append_element] } {
		set to_append $append_element
	} else {
		set to_append "Ad $ad_id"
	}

	#
	# We call categories::generate_trail with "-append_element foo" so we 
	# can cache its results effectively.
	#
	
	set trail_list [util_memoize "classified-ads::categories::generate_trail -keyword_id $keyword_id -append_element foo"]

	set last_element_index [expr [llength $trail_list] - 1]
	set trail_list [lreplace $trail_list $last_element_index $last_element_index $to_append]

	return $trail_list
}
