ad_library {

Classified Ads Procs

@author Deds Castillo (deds@infiniteinfo.com)
@creation-date 2002-10-08
@cvs-id $Id: classified-ads-procs.tcl,v 1.15 2003/11/07 20:01:19 rmello Exp $

}

namespace eval classified-ads {}
ad_proc -public classified-ads::package_key {} {
	Returns the package key
} {
	return "classified-ads"
}


ad_proc -public classified-ads::is_instantiated {} {
	Returns 1 if classified-ads is instantiated, 0 otherwise
} {
	return [ad_decode [apm_num_instances [package_key]] 0 0 1]
}


ad_proc -public classified-ads::get_main_keyword_id {} {
	Returns the keyword id of the main category for the package, 0 if not yet created
} {
	# These are magic values
	set heading "[package_key]-main-category"
	set description "$heading Main Category, this is where all classified ads categories are rooted"

	return [db_string get_main_keyword {} -default 0]
}


ad_proc -public classified-ads::root_folder_exists_p {} {
	Returns the folder id of the classified ads root folder, 0 if not yet created
} {
	# These are magic values
	set name "[package_key]"
	set label "[package_key]"
	set description "$name Root Folder, all classified ads items go here"

	return [db_string get_root_folder {} -default 0]

}

ad_proc -public classified-ads::get_root_folder_id {} {
	Returns the folder id of the classified ads root folder, error if none
} {
	set folder_id [root_folder_exists_p]

	if {!$folder_id} {
		ad_return_error "Root Folder not found" \
			"The root folder for [package_key] was not found. The datamodel might not be properly installed.  Did you restart the server after installation?"
		ad_script_abort
	}

	return $folder_id
}

ad_proc -private classified-ads::install_sitewide_attributes {
	-package_id:required
} {
	Checks if the site-wide attributes (used by all classified-ads
	instances) have been installed and installs them if necessary.
	Called through an after-mount Tcl callback. We do this here instead
	of in the SQL creation scripts because we need the keyword_id of the
	root classified-ads category (because we now support per-category
	custom fields).
} {
	set main_keyword_id [classified-ads::get_main_keyword_id]
	set attributes [list type expires_on price]

	foreach attrib $attributes {
		set count [db_1row count_attributes {}]	

		if { $count != 1 } {
			#
			# This site-wide attribute has not been created. Create them.
			#

			db_transaction {
				switch $attrib {
					"type" {
						set attribute_name "type"
						set datatype "integer"
						set pretty_name "Type"
						set pretty_plural "Types"
						set default_value 0
						set column_spec "integer"
						set widget "radio"
						set html_options ""
						set extra_options ""
						set select_options "{{\"For Sale\" 0} {\"Wanted to Buy\" 1}}"
						set validate ""
						set optional_p "f"
						set enabled_p "t"
					}
					"expires_on" {
						set attribute_name "expires_on"
						set datatype "date" 
						set pretty_name "Expires On"
						set pretty_plural "Expires On" 
						set default_value "" 
						set column_spec "timestamp"
						set widget "date" 
						set html_options ""
						set extra_options " -format \"MONTH DD YYYY\" -help" 
						set select_options ""
						set validate ""
						set optional_p "t" 
						set enabled_p "t"
					}
					"price" {
						set attribute_name "price"
						set datatype "string"
						set pretty_name "Price"
						set pretty_plural "Prices"
						set default_value 0
						set column_spec "numeric(9,2)"
						set widget "currency"
						set html_options ""
						set extra_options ""
						set select_options ""
						set validate "{ price { \[expr \$value < 0\] } { \"Price has to be positive.\" }}"
						set optional_p "f"
						set enabled_p "t"
					}
				}

				set attribute_id [db_exec_plsql create_attribute {}]
				db_exec_plsql register_widget {}
			}
		}

	}

	return 1 
}

ad_proc -public classified-ads::get_folder_id {
	{-package_id}
} {
	Returns the folder_id for an instance of classified ads

	@param package_id The package id of this classified ads instance. 
 			  Default to current instance if none passed.
} {
	if {![exists_and_not_null package_id]} {
		set package_id [ad_conn package_id]
	}

	return [db_string select_folder_id {}]
}


ad_proc -public classified-ads::get_mounted_packages {} {
	Returns a list of mounted classified-ads packages
} {
	set package_key [package_key]
	return [db_list select_packages {}]
}

ad_proc -public classified-ads::get_content_type {
	{-item_id:required}
} {
	Gets the content type of a given item
} {
	return [db_string select_type {}]
}


ad_proc -public classified-ads::get_table_name {
	{-content_type:required}
} {
	Gets the table name of one content type
} {
	return [db_string select_table_name {}]
}

ad_proc -public classified-ads::register_content_type {
	{-folder_id:required}
	{-content_type:required}
	{-include_subtypes "f"}
} {
	Registers a given content type to a particular folder
} {
	return [db_list register_content_type {}]
}

ad_proc -public classified-ads::create_package_folder {
	{-root_folder_id:required}
	{-package_id:required}
} {
	Creates a content revision folder for one instance of classified ads

	@param root_folder_id The root folder id for the whole classified ads package
	@param package_id The package id of the instance whose folder is to be created
} {
	set name "[package_key]-$package_id"
	set label "$name folder"
	set description "Container folder for one instance of classified-ads"

	db_transaction {
		set new_folder_id [db_exec_plsql create_folder {}]
		db_exec_plsql associate_package {}
	}

	return $new_folder_id
}

ad_proc -public classified-ads::create_ca_item {
	{-item_id_element:required}
	{-parent_id}
	{-category_id}
	{-content_type:required}
	{-form_id:required}
	{-title_column "title"}
	{-description_column "description"}
	{-data_column "data"}
} {
	Creates a classified ads item.  This is generic enough to:
	1. handle items that are either derived from content_revision or not
	2. dynamically generate insert statements for extension tables
	3. dynamically generate insert statements to handle generic attributes
	4. create new revisions as appropriate

	@author Deds Castillo
	@author Roberto Mello (rmello@fslc.usu.edu)

	@param item_id_element The name of the form element that should be used as the item id
	@param content_type The type of the item being inserted
	@param form_id The ID of the form that contains the the name/value pairs

} {
	set package_id [ad_conn package_id]

	if {![info exists parent_id]} {
		set parent_id [get_folder_id -package_id $package_id]
	}

	set creation_user [ad_conn user_id]
	set creation_ip [ad_conn peeraddr]

	# DEDS: FIXME - for now we are assuming required fields are filtered
	# out by the validation in the form procedures.  i think we should
	# double check it here again and barf if one of them is missing

	# get the item id, barf if not present

	if {[template::element::exists $form_id $item_id_element]} {
		set item_id [template::element::get_value $form_id $item_id_element]
	} else {
		ad_return_complaint 1 "<li> Passed in item id not found"
		ad_script_abort
	}

	# check if this is a subclass of content_revision

	set cr_subclass_p [is_subclass -content_type $content_type -parent_type content_revision]
	set content_item [get_supertype -content_type $content_type]

	set name "$content_type --  $item_id"
	set title [template::element::get_value $form_id $title_column]

	if {[empty_string_p $title]} {
		# make sure there is always a title
		set title $name
	}

	set description [template::element::get_value $form_id $description_column]

	if {[template::element::exists $form_id $data_column]} {
		set data [template::element::get_value $form_id $data_column]
	}

	# do everything in a transaction

	db_transaction {
		# insert the content item

		set object_id [db_exec_plsql insert_content_item {}]
		set cr_item_id $object_id

		if {$cr_subclass_p} {
			set object_id [db_exec_plsql insert_content_revision {}]
		}

		# RBM: If a category was passed, assign the ad to it.

		if {[info exists category_id]} {
			db_exec_plsql assign_item_to_category {}
		}

		# DEDS: FIXME - this only supports one table
		#   fix it so that it can span multiple tables
		#
		# get the type specific attributes

		set type_specific_attr_list [get_attributes -content_type $content_type -storage "type_specific"]
	
		if {[llength $type_specific_attr_list]} {
			# we always reference the item_id
			set name_list [list $item_id_element]
			set value_list [list $object_id]
			set type_specific_bind_vars [ns_set create]
		
			foreach one_attribute $type_specific_attr_list {
				set column_name [ns_set get $one_attribute column_name]
				set table_name [ns_set get $one_attribute table_name]
		
				if {[template::element::exists $form_id $column_name]} {
					lappend name_list $column_name
					lappend value_list ":$column_name"
					set one_value [template::element::get_value $form_id $column_name]

					if {[string match "date" [ns_set get $one_attribute datatype]]} {
						set one_value [classified-ads::widgets::date_widget_to_sql -date $one_value]
					}

					ns_set put $type_specific_bind_vars $column_name $one_value
			
				} else {
					set default_value [ns_set get $one_attribute default_value]

					if {![empty_string_p $default_value]} {
						lappend name_list $column_name
						lappend value_list ":$column_name"
						ns_set put $type_specific_bind_vars $column_name $default_value
					}
				}
			}
		
			# create the insert statement for type specific attributes
		
			set plsql_query "insert into $table_name ([join $name_list ", "]) values ([join $value_list ", "])"

			# insert the type specific data

			db_dml insert_data {} -bind $type_specific_bind_vars
			ns_set free $type_specific_bind_vars
		}

		# get the generic attributes

		set generic_attr_list [get_attributes -content_type $content_type -storage "generic"]
	
		if {[llength $generic_attr_list]} {
	
			foreach one_attribute $generic_attr_list {
				set attribute_id [ns_set get $one_attribute attribute_id]
				set column_name [ns_set get $one_attribute column_name]
				set datatype [ns_set get $one_attribute datatype]
		
				if {[template::element::exists $form_id $column_name]} {
					set attr_value [template::element::get_value $form_id $column_name]
			
					if {[string match "date" $datatype]} {
						set attr_value [classified-ads::widgets::date_widget_to_sql -date $attr_value]
					}
				} else {
					set default_value [ns_set get $one_attribute default_value]

					if {![empty_string_p $default_value]} {
						set attr_value $default_value
					} else {
						set attr_value ""
					}
				}

				insert_generic_value -object_id $item_id -attribute_id $attribute_id -attr_value $attr_value
			}
		}
	} on_error {
		ad_return_complaint 1 "<li>$errmsg"
		ad_script_abort
	}

	return $item_id

}

ad_proc -public classified-ads::edit_ca_item {
	{-item_id_element:required}
	{-form_id:required}
	{-title_column "title"}
	{-description_column "description"}
	{-data_column "data"}
	{-revision_id}
} {
	Edits a classified ads item.

	@param item_id_element The name of the form element that should be used as the item id
	@param form_id The ID of the form that contains the the name/value pairs
	@param title_column The element name in the form that will be used as title of this content
	@param description_column The element name in the form that will be used as description of this content
} {

	set creation_user [ad_conn user_id]
	set creation_ip [ad_conn peeraddr]

	# DEDS: FIXME - for now we are assuming required fields are filtered
	# out by the validation in the form procedures.  i think we should
	# double check it here again and barf if one of them is missing

	# get the item id, barf if not present

	if {[template::element::exists $form_id $item_id_element]} {
		set item_id [template::element::get_value $form_id $item_id_element]
	} else {
		ad_return_error "Passed item_id not found" "<li> Passed in item id not found."
		ad_script_abort
	}

	set create_revision_p 0

	# check if this is a subclass of content_revision

	set content_type [get_content_type -item_id $item_id]
	set cr_subclass_p [is_subclass -content_type $content_type -parent_type content_revision]

	if {$cr_subclass_p} {
		if {[parameter::get -parameter create_new_ad_revision_p -default 0]} {
			set create_revision_p 1
		}
		if {[info exists revision_id]} {
			set object_id $revision_id
		} else {
			set object_id [get_latest_revision -item_id $item_id]
		}

	} else {
		set object_id $item_id
	}

	set name "$content_type --  $item_id"
	set title [template::element::get_value $form_id $title_column]

	#
	# RBM: FIXME? This doesn't look like what should be done to me.
	#

	if {[empty_string_p $title]} {
		# make sure there is always a title
		set title $name
	}
	
	set description [template::element::get_value $form_id $description_column]

	if {[template::element::exists $form_id $data_column]} {
		set data [template::element::get_value $form_id $data_column]
	}

	# do everything in a transaction
	
	db_transaction {
		#
		# DEDS: FIXME - this only supports one table
		#   fix it so that it can span multiple tables

		if {$create_revision_p} {
			set object_id [db_exec_plsql insert_content_revision {}]

		} elseif {$cr_subclass_p} {
			# DEDS: FIXME - this is straghtforward update.  we should be using cr api.
			db_dml update_content_revision {}
		}

		# Get the type specific attributes
		
		set type_specific_attr_list [get_attributes -content_type $content_type -storage "type_specific"]
	
		if {[llength $type_specific_attr_list]} {
			#
			# We always reference the item_id
			#

			set update_list [list]
			set name_list [list $item_id_element]
			set value_list [list $object_id]
			set type_specific_bind_vars [ns_set create]
		
			foreach one_attribute $type_specific_attr_list {
				set column_name [ns_set get $one_attribute column_name]
				set table_name [ns_set get $one_attribute table_name]
				set datatype [ns_set get $one_attribute datatype]

				if {[template::element::exists $form_id $column_name]} {

					lappend name_list $column_name
					lappend value_list ":$column_name"
					lappend update_list "$column_name = :$column_name"
					set one_value [template::element::get_value $form_id $column_name]

					if {[string match "date" $datatype]} {
						set one_value [classified-ads::widgets::date_widget_to_sql -date $one_value]
					}
						ns_set put $type_specific_bind_vars $column_name $one_value
				}
			}

			ns_set put $type_specific_bind_vars item_id $object_id

			# create the update statement for type specific attributes

			if {$create_revision_p} {
				set plsql_query "insert into $table_name ([join $name_list ", "]) values ([join $value_list ", "])"
			} else {
				set plsql_query "update $table_name set [join $update_list ", "] where $item_id_element = :item_id"
			}

			# update the type specific data
	
			db_dml update_data {} -bind $type_specific_bind_vars
			ns_set free $type_specific_bind_vars
		}

		# get the generic attributes
	
		set generic_attr_list [get_attributes -content_type $content_type -storage "generic"]
	
		if {[llength $generic_attr_list]} {
			foreach one_attribute $generic_attr_list {
				set attribute_id [ns_set get $one_attribute attribute_id]
				set column_name [ns_set get $one_attribute column_name]
				set datatype [ns_set get $one_attribute datatype]

				if {[template::element::exists $form_id $column_name]} {
					set attr_value [template::element::get_value $form_id $column_name]

					if {[string match "date" $datatype]} {
						set attr_value [classified-ads::widgets::date_widget_to_sql -date $attr_value]
					}
				}

				if {$create_revision_p} {
					insert_generic_value -object_id $object_id -attribute_id $attribute_id -attr_value $attr_value
				} else {
					update_generic_value -object_id $object_id -attribute_id $attribute_id -attr_value $attr_value
				}

			}
		}
	} on_error {
		ad_return_error "Problems" "<li> $errmsg"
		ad_script_abort
	}

	return $object_id
}


ad_proc -public classified-ads::is_subclass {
	{-content_type:required}
	{-parent_type:required}
} {
	Returns 1 if an object type is a subclass of another

	@param content_type The object type to check
	@param parent_type The supertype class the proc test against
} {
	return [ad_decode [db_string test_subclass {}] f 0 1]
}


ad_proc -public classified-ads::get_attributes {
	{-content_type "ca_ad"}
	{-keyword_id}
	{-storage "all"}
} {
	Returns a list of ns_sets of the attributes for one content type

	@param content_type This is the content type whose attributes we get

	@param keyword_id Only the attributes belonging to this category are retrieved.
	                  Defaults to current package's main category.

	@param storage Only the attributes who belong to this storage type are retrieved.
		       Can be "type_specific", "generic", or "all".
} {
	if {![exists_and_not_null keyword_id]} {
		set package_id [ad_conn package_id]
		set keyword_id [util_memoize "classified-ads::categories::get_package_keyword_id -package_id $package_id"]
	}

	if {![string equal $storage "all"]} {
		set storage_stub "and storage = '$storage'"
	} else {
		set storage_stub ""
	} 

	return [db_list_of_ns_sets select_attributes {}]
}

ad_proc -public -deprecated classified-ads::get_attribute_names {
	{-item_id}
	{-storage "all"}
} {
	Returns a list of lists of the attribute names for one item.

	@param package_id The item id  whose attribute names we get
} {
	#
	# RBM: I don't understand why we need to support more than one
	#      content_type and all the associated complications.
	#      This proc was only used in one place, and I didn't see
	#      a reason to not deprecate it until I figure out why.
	#

	if {![exists_and_not_null package_id]} {
		set package_id [ad_conn package_id]
	}

	if {![string equal $storage "all"]} {
		set storage_stub "and storage = '$storage'"
	} else {
		set storage_stub ""
	} 

	set content_type [get_content_type -item_id $item_id]
	return [db_list_of_lists select_names {}]
}


ad_proc -public classified-ads::get_attribute_values {
	{-item_id:required}
	{-item_id_element:required}
	{-storage "all"}
	{-revision_id}
} {
	Returns a list of lists of the attribute values for one item.

	@param item_id The ID of the object whose values we retrieve
	@param storage Only the attribute values who belong to this storage type are retrieved.
	               Can be "type_specific", "generic", or "all".
} {
	#
	# RBM: FIXME: This looks too messy. Find out reason for all this
	#             and try to simplify.
	#

	set values_list [list]

	set cr_subclass_p [is_subclass -content_type [get_content_type \
					-item_id $item_id] \
					-parent_type content_revision]

	if {![info exists revision_id] && $cr_subclass_p} {
		set revision_id [get_latest_revision -item_id $item_id]
	}

	if {[string match $storage "type_specific"] || [string match $storage "all"]} {
		set table_name [get_table_name -content_type [get_content_type -item_id $item_id]]
		# set names_value_list [get_attribute_names -item_id $item_id -storage type_specific]
		set keyword_id [classified-ads::ads::get_category_id -ad_id $item_id]
		set names_value_list \
			[classified-ads::categories::get_custom_fields_and_datatypes -keyword_id $keyword_id]
		set names_list [list]

		foreach one_name_value $names_value_list {
			lappend names_list [lindex $one_name_value 0]
		}

		if {$cr_subclass_p} {
			set query_string "select [join $names_list ", "]
				from $table_name
				where $item_id_element = '$revision_id'"
		} else {
			set query_string "select [join $names_list ", "]
				from $table_name
				where $item_id_element = '$item_id'"
		}

		db_1row select_values {} -column_array type_specific_row

			foreach one_name_value $names_value_list {
				set datatype [lindex $one_name_value 1]
				set one_value $type_specific_row([lindex $one_name_value 0])

				if {[string match "date" $datatype] && ![empty_string_p $one_value]} {
					set one_value [classified-ads::widgets::sql_to_date_widget -sql_date $one_value -format "YYYY-MM-DD"]
				}

				lappend values_list [list [lindex $one_name_value 0] $one_value]
			}
	}

	# RBM: FIXME - We shouldn't generate the entire query here?

	if {[string match $storage "generic"] || [string match $storage "all"]} {
		if {$cr_subclass_p} {
			set query_string "select coalesce(aa.column_name, aa.attribute_name) as column_name,
	                                         cav.attr_value,
			                         aa.datatype
				            from acs_attributes aa,
			                         ca_attribute_values cav
				           where aa.attribute_id = cav.attribute_id and
				                 cav.object_id = '$revision_id'"
		} else {
			set query_string "select coalesce(aa.column_name, aa.attribute_name) as column_name,
			                         cav.attr_value,
			                         aa.datatype
				            from acs_attributes aa,
			                         ca_attribute_values cav
				           where aa.attribute_id = cav.attribute_id and
				                 cav.object_id = '$item_id'"
		}

		set generic_list [db_list_of_lists select_values {}]

		foreach one_name $generic_list {
			set datatype [lindex $one_name_value 2]
			set one_value [lindex $one_name_value 1]

			if {[string match "date" $datatype] && ![empty_string_p $one_value]} {
				set one_value [classified-ads::widgets::sql_to_date_widget -sql_date $one_value -format "YYYY-MM-DD"]
			}
			lappend values_list [list [lindex $one_name 0] $one_value]
		}
	}

	return $values_list
}

ad_proc -public classified-ads::get_supertype {
	{-content_type:required}
} {
	Returns the direct ancestor of one content type

	@param content_type This is the content type whose ancestor we get
} {
	return [db_string select_supertype {}]
}


ad_proc -public classified-ads::insert_generic_value {
	{-object_id:required}
	{-attribute_id:required}
	{-attr_value ""}
} {
	Inserts one value for a generic attribute

	@param object_id The object id of the objects who owns this value
	@param attribute_id The id of the attribute that will contain a new value
	@param attr_value The associated value of this attribute for this object
} {
	return [db_exec_plsql insert_value {}]
}


ad_proc -public classified-ads::update_generic_value {
	{-object_id:required}
	{-attribute_id:required}
	{-attr_value ""}
} {
	Updates one value for a generic attribute

	@param object_id The object id of the objects who owns this value
	@param attribute_id The id of the attribute that will be updated
	@param attr_value The associated value of this attribute for this object
} {
	return [db_exec_plsql update_value {}]
}


ad_proc -public classified-ads::get_latest_revision {
	{-item_id:required}
} {
	Returns the latest revision for one item

	@param item_id The id of the item that we want to get the latest revision
} {
	return [db_string get_revision {} -default 0]
}


ad_proc -public classified-ads::get_content {
	{-revision_id:required}
} {
	Returns the content of one item

	@param revision_id The id of the revision to get the data from
} {
	return [db_string select_content {}]
}


ad_proc -public classified-ads::context_bar {
	{-node_id}
	args
} {
	Returns a context bar relative to classified ads

	@param node_id if provided, then we work up from this node
} {
	if { ![info exists node_id] } {
		set node_id [ad_conn node_id]
	}

	set context [list]

	db_foreach context {} { 
		lappend context [list $url $object_name]
	}

	if { [string match admin/* [ad_conn extra_url]] } {
		lappend context [list "[ad_conn package_url]admin/"  "Administration"]
	}

	if {[llength $args] == 0} { 
		# fix last element to just be literal string

		set context [lreplace $context end end [lindex [lindex $context end] 1]]
	}

	return [ad_context_bar_html [concat $context $args]]
}

ad_proc classified-ads::validate_isbn { isbn } {
	Does a simple check-digit verification of isbn.
	@author Ben Bytheway
} { 
	set isbn [string trim $isbn]
	regsub -all {(\D&^Xi&^x)} $isbn "" isbn

	if {![regexp {^(\d|X|x){10}$} $isbn]} {
		return 1 
	}

	set check 0
	set test_isbn [split $isbn ""]

	if {[lindex $test_isbn end] == "x" || [lindex $test_isbn end] == "X"} {
		set test_isbn [lreplace $test_isbn end end 10 ]
	}

	for {set i 0} {$i <= 9} {incr i} {
		set digit [lindex $test_isbn $i]
		set check [expr ($digit * (10 - $i)) + $check]
	}

	set check [expr $check % 11]

	if { !$check && [string index $isbn 0] != 6} {
		return 0 
	} else {
		return 1  
	}
}
