# /packages/cop-base/tcl/cop-procs.tcl
ad_library {
    TCL library for the COP package

    @author Jeff Davis <davis@xarg.net>

    @creation-date 2003-10-30
    @cvs-id $Id: cop-procs.tcl,v 1.8 2005/03/25 08:40:29 jeffd Exp $
}

namespace eval cop {}
namespace eval cop::util {}

ad_proc -public cop::util::url {
    {-node_id {}}
} {
    Returns the cop-ui URL for a given subsite.

    @param subsite_id the subsite for which to find the cop-ui url

    @return a url for the cop-ui package

    @author Jeff Davis davis@xarg.net
    @creation-date 2003-10-30
} {
    if {[empty_string_p $node_id]} { 
        set node_id [site_node::closest_ancestor_package -package_key [subsite::package_keys] -node_id [ad_conn node_id] -include_self -element node_id]
    }

    return [lindex [site_node::get_children -node_id $node_id -package_key cop-ui -all] 0]
}

ad_proc -public cop::util::package_id {
    {-node_id {}}
} {
    Returns the first cop-ui package under node_id,
    node_id defaults to the nearest acs-subsite.

    @param subsite_id the subsite for which to find the cop-ui package_id

    @return a url for the cop-ui package

    @author Jeff Davis davis@xarg.net
    @creation-date 2003-10-30
} {
    if {[empty_string_p $node_id]} {
        set node_id [site_node::closest_ancestor_package -package_key acs-subsite -node_id [ad_conn node_id] -include_self -element node_id]
    }

    return [lindex [site_node::get_children -element package_id -node_id $node_id -package_key cop-ui -all] 0]
}


ad_proc -private cop::load_install_xml {filename binds} {
    Loads an install file and returns the root node.
    errors out if the file is not there.

    substitutes variables before parsing so you can provide interpolated values.

    @param filename relative to serverroot, leading slash needed.
    @param binds list of {variable value variable value ...}

    @return root_node of the parsed xml file.

    @author Jeff Davis davis@xarg.net
    @creation-date 2003-10-30
} {
    # Abort if there is no install.xml file
    set filename [acs_root_dir]$filename

    if { ![file exists $filename] } {
        error "File $filename not found"
    }

    # Read the whole file
    set file [open $filename]
    set __the_body__ [read $file]
    close $file
    # Interpolate the vars.
    if {![empty_string_p $binds]} {
        foreach {var val} $binds {
            set $var [ad_quotehtml $val]
        }
        if {![info exists Id]} {
            set Id {$Id}
        }
        if {[catch {set __the_body__ [subst -nobackslashes -nocommands ${__the_body__}]} err]} {
            error $err
        }
    }

    set root_node [xml_doc_get_first_node [xml_parse -persist ${__the_body__}]]
    return $root_node
}

ad_proc -private cop::install {filename binds} {    set root_node [cop::load_install_xml $filename $binds] 
    install a CoP subsite from an xml definition.

    @parameter filename path to the xml file defining the CoP relative to serverroot.
    @param binds list of {variable value variable value ...}

    @return list of messages

    @author Jeff Davis (swiped from acs-bootstrap-installer though)
    @creation-date 2003-10-30
} {
    set root_node [cop::load_install_xml $filename $binds] 

    set acs_application(name) [apm_required_attribute_value $root_node name]
    set acs_application(pretty_name) [apm_attribute_value -default $acs_application(name) $root_node pretty-name]

    lappend out "Loading packages for the $acs_application(pretty_name) application."

    set actions [xml_node_get_children_by_name $root_node actions]

    if { [llength $actions] != 1 } {
        ns_log Error "Error in \"$filename\": only one action node is allowed, found: [llength $actions]"
        error "Error in \"$filename\": only one action node is allowed"
    }

    set actions [xml_node_get_children [lindex $actions 0]]

    foreach action $actions {
        switch -exact [xml_node_get_name $action] {
            text {}

            install {
                set install_spec_files [list]
                foreach install_spec_file [glob -nocomplain "[acs_root_dir]/packages/[apm_required_attribute_value $action package]/*.info"] {
                    if { [catch { array set package [apm_read_package_info_file $install_spec_file] } errmsg] } {
                        # Unable to parse specification file.
                        error "install: $install_spec_file could not be parsed correctly.  The error: $errmsg"
                        return
                    }

                    if { [apm_package_supports_rdbms_p -package_key $package(package.key)]
                         && ![apm_package_installed_p $package(package.key)] } {
                        lappend install_spec_files $install_spec_file
                    }
                }

                set pkg_info_list [list]
                foreach spec_file [glob -nocomplain "[acs_root_dir]/packages/*/*.info"] {
                    # Get package info, and find out if this is a package we should install
                    if { [catch { array set package [apm_read_package_info_file $spec_file] } errmsg] } {
                        # Unable to parse specification file.
                        error "install: $spec_file could not be parsed correctly.  The error: $errmsg"
                    }

                    if { [apm_package_supports_rdbms_p -package_key $package(package.key)]
                         && ![apm_package_installed_p $package(package.key)] } {
                        # Save the package info, we may need it for dependency satisfaction later
                        lappend pkg_info_list [pkg_info_new $package(package.key) $spec_file \
                                                   $package(provides) $package(requires) ""]
                    }
                }

                if { [llength $install_spec_files] > 0 } {
                    set dependency_results [apm_dependency_check -pkg_info_all $pkg_info_list $install_spec_files]
                    if { [lindex $dependency_results 0] == 1 } {
                        apm_packages_full_install -callback apm_ns_write_callback [lindex $dependency_results 1]
                    } else {
                        foreach package_spec [lindex $dependency_results 1] {
                            if { [string is false [pkg_info_dependency_p $package_spec]] } {
                                append err_out "install: package \"[pkg_info_key $package_spec]\"[join [pkg_info_comment $package_spec] ","]\n"
                            }
                        }
                        error $err_out
                    }
                }
            }

            mount {

                set package_key [apm_required_attribute_value $action package]
                set instance_name [apm_required_attribute_value $action instance-name]
                set mount_point [apm_required_attribute_value $action mount-point]

                regexp {(.*)/([^/]*)$} $mount_point match parent_url mount_point
                if {[string eq $parent_url ""]} { 
                    set parent_url /
                }
                set parent_id [site_node::get_node_id -url $parent_url]

                if { [catch {
                    db_transaction {
                        set node_id [site_node::new -name $mount_point -parent_id $parent_id]
                    }
                } error] } {
                    # There is already a node with that path, check if there is a package mounted there
                    error $error
                    array set node [site_node::get -url "/$mount_point"]
                    if { [empty_string_p $node(object_id)] } {
                        # There is no package mounted there so go ahead and mount the new package
                        set node_id $node(node_id)
                    } else {
                        error "A package is already mounted at \"$mount_point\""
                    }
                }

                if { ![empty_string_p $node_id] } {
                    lappend out "Mounting new instance of package $package_key at /$mount_point"
                    site_node::instantiate_and_mount \
                        -node_id $node_id \
                        -node_name $mount_point \
                        -package_name $instance_name \
                        -package_key $package_key

                }

            }

            set-parameter {
                set name [apm_required_attribute_value $action name]
                set value [apm_required_attribute_value $action value]
                set package_key [apm_attribute_value -default "" $action package]
                set url [apm_attribute_value -default "" $action url]

                if { ![string equal $package_key ""] && ![string equal $url ""] } {
                    error "set-parameter: Can't specify both package and url for $url and $package_key"
                } elseif { ![string equal $package_key ""] } {
                    parameter::set_from_package_key -package_key $package_key -parameter $name -value $value
                } else {
                    parameter::set_value \
                        -package_id [site_node::get_object_id -node_id [site_node::get_node_id -url $url]] \
                        -parameter $name \
                        -value $value
                }
            }

            default {
                error "Error in \"$filename\": got bad node \"[xml_node_get_name $action]\""
            }

        }

    }
    return $out
}

ad_proc -private cop::util::packages_no_mem {
    -node_id
} {
    return a list of packages for the subsite containing node_id

    @author Jeff Davis davis@xarg.net
    @creation-date 2004-05-07
    @see cop::util::packages
} {
    # need to strip nodes which have no mounted package...
    set packages [list]
    foreach package [site_node::get_children -all -node_id $node_id -element package_id] {
        if {![empty_string_p $package]} {
            lappend packages $package
        }
    }

    return $packages
}

ad_proc -public cop::util::packages {
    -node_id
} {
    Return a list of packages for the subsite containing node_id

    Memoized function.

    @author Jeff Davis davis@xarg.net
    @creation-date 2004-05-07
    @see cop::util::packages_no_mem
} {
    set subsite_node_id [site_node::closest_ancestor_package \
                             -package_key acs-subsite \
                             -node_id $node_id \
                             -include_self \
                             -element node_id]

    return [util_memoize [list cop::util::packages_no_mem -node_id $subsite_node_id] 1200]
}

ad_proc -private cop::util::catmap {
    -name
    -description
    -context_id
    -cats
} {
    set tree_id [category_tree::add -name $name -description $description -context_id $context_id]

    set parent(0) {}
    set parent(1) {}
    set parent(2) {}

    foreach {cat key} $cats {
        set parent([expr $cat + 1]) [category::add -noflush -name $key -description $key -tree_id $tree_id -parent_id $parent($cat)]
    }

    category_tree::flush_cache $tree_id
    category_tree::map -tree_id $tree_id -object_id $context_id

    ns_log Notice "cop::util::catmap $name tree id $tree_id created and mapped to $context_id"

    return $tree_id
}

ad_proc -private cop::util::after_mount {
    -package_id
    -node_id
} {
    Create the KM  categories and map them to mainsite
} {
    set cats {
        0 KM
        1 {Urgent question}
        1 {Key document}
        1 {Unanswered question}
        1 {Answered question}
        1 {Discussion}
    }

    set tree_id [cop::util::catmap -name KM -description "KM categories" -context_id [subsite::main_site_id] -cats $cats]

    parameter::set_value -package_id $package_id -parameter KMCategoryTreeID -value $tree_id
}

ad_proc -private cop::util::after_ui_mount {
    -package_id
    -node_id
} {
    Map the KM categories for the cop-base instance to the subsite containing the cop-ui package

    WARNING: This is a callback in cop-ui not cop-base.
} {
    set subsite_id [site_node::closest_ancestor_package -package_key acs-subsite -node_id $node_id -element package_id]

    set base_id [apm_package_id_from_key cop-base]

    set tree_id [parameter::get -package_id $base_id -parameter KMCategoryTreeID]

    category_tree::map -tree_id $tree_id -object_id $subsite_id
}
