ad_page_contract {

    Add an arbitrary number of groups with associated relational segments and
    their members.

    @author Don Baccus (dhogaza@pacifier.com)
    @creation-date 2003-01-07

}

set package_id [ad_conn package_id]
set user_id [ad_conn user_id]
set creation_ip [ad_conn peeraddr]

permission::require_permission -party_id $user_id -object_id $package_id -privilege admin

db_1row get_group_count {}
db_1row get_user_count {}

set membership_rels [db_list_of_lists get_membership_rels {}]

ad_form -name add_groups -form {
    {new_groups:integer        {label "Number of new groups to add?"}
                               {html { size 5 }}}
    {rel_segments:text(multiselect),multiple,optional
                               {label "Relational segments to add"}
                               {options $membership_rels}}
    {group_users:integer       {label "Number of users to add to each group (max $current_users)?"}
                               {html { size 5 }}
                               {value 0}}
} -validate {
    {group_users
        { $group_users <= $current_users }
        "You can only add $current_users users to each group"
    }
} -on_submit {

    # DRB: some notes for those not familiar with relational segments.

    # A relational segment defines a subset of a group by mapping a
    # group_id to a acs_relations rel_type.  This script defines a default
    # relational segment of rel_type membership_rel that contains all members
    # of the group.  Logically there's a 1:1 relationship between members of the
    # group and members of the membership relational segment (think "subset") of the
    # group.  Therefore we don't need to define a new relation to map users
    # to the membership relational segment.  Though each user is visible in the
    # group member map and relational segment member map, each occupies a single
    # row in acs_rels.  Sweet!  This trick was used by aD in acs-subsite.

    # Therefore each user appears once in acs_rels to map group membership and
    # the "Group Name Members" relational segment, and once more for each optional
    # relational segment of type other than membership_rel.

    # You may think at first that sharing acs_rels rows between the group itself
    # and membership_rels isn't general and therefore is a bad idea.  What if
    # another relational segment of membership_rel is defined that's meant to hold
    # a subset of the groups members?

    # You can't do this as the datamodel for relational segments constrains
    # (group_id, rel_type) to be unique.  Therefore our default membership_rel
    # relational segment is the only one that can be created and we're guaranteed
    # everything will work.

    # This makes sense as the additional relational segments defined for the group
    # are logically subsets that may or may not have a 1:1 relationship with the members
    # of the parent group.  Therefore we couldn't really "poach" the membership_rel relation
    # that was defined for the parent group even if the unique constraint were relaxed
    # (as aD hackers suggested might be reasonble).  

    # Why create the members relational segment since the group already makes
    # use of the membership_rel that's been created for it?  Well ... for one thing
    # membership checks and permissions checking can ignore groups and only check
    # relational segments if this is done consistently in the toolkit.  Code that
    # lists all members can be shared with code that lists all (say) students rather
    # than be a special case script that is driven by the group rather than a subset
    # of the group.  The list goes on.  Using relational segments consistently will
    # simplify and with luck speed code ...

    for { set i 1 } { $i <= $new_groups } { incr i } {
         incr current_groups

         db_transaction {
             set group_name "Membership Group $current_groups"
             set group_id [group::new -group_name $group_name group]
	     set rels [list membership_rel]
             rel_segments_new $group_id membership_rel "$group_name Members"
             foreach type_and_plural $rel_segments {
                 set type_and_plural [split $type_and_plural ":"]
                 set rel_type [lindex $type_and_plural 0]
                 lappend rels $rel_type
                 rel_segments_new $group_id $rel_type "$group_name [lindex $type_and_plural 1]"
             }
         } on_error {
             ad_return_error "Error creating group \"$group_name\"" $errmsg
             ad_script_abort
         }

         db_multirow get_users get_users {} {
             db_transaction {
               foreach rel $rels {
                   relation_add -member_state approved $rel $group_id $user_id
               }
             } on_error {
                 ad_return_error "Error adding users to group \"$group_name\"" $errmsg
                 ad_script_abort
             }
         }
    }

    ad_returnredirect "index"
    ad_script_abort

}

set context {"Add groups"}

ad_return_template
